package post_deals_worker

import (
	"context"
	"database/sql"
	"encoding/json"
	"errors"
	"fmt"
	"go.uber.org/zap"
	"penahub.gitlab.yandexcloud.net/backend/quiz/bitrix/internal/models"
	"penahub.gitlab.yandexcloud.net/backend/quiz/bitrix/internal/tools"
	"penahub.gitlab.yandexcloud.net/backend/quiz/bitrix/pkg/bitrixClient"
	"penahub.gitlab.yandexcloud.net/backend/quiz/common.git/dal"
	"penahub.gitlab.yandexcloud.net/backend/quiz/common.git/model"
	"penahub.gitlab.yandexcloud.net/backend/quiz/common.git/pj_errors"
	"penahub.gitlab.yandexcloud.net/backend/quiz/common.git/repository/bitrix"
	"penahub.gitlab.yandexcloud.net/backend/quiz/common.git/utils"
	"strconv"
	"strings"
	"time"
)

type Deps struct {
	BitrixRepo   *dal.BitrixDal
	BitrixClient *bitrixClient.Bitrix
	Logger       *zap.Logger
}

type DealsWorker struct {
	bitrixRepo   *dal.BitrixDal
	bitrixClient *bitrixClient.Bitrix
	logger       *zap.Logger
}

func NewDealsWC(deps Deps) *DealsWorker {
	return &DealsWorker{
		bitrixRepo:   deps.BitrixRepo,
		bitrixClient: deps.BitrixClient,
		logger:       deps.Logger,
	}
}

func (wc *DealsWorker) Start(ctx context.Context) {
	ticker := time.NewTicker(1 * time.Minute)
	defer ticker.Stop()

	for {
		select {
		case <-ticker.C:
			wc.startFetching(ctx)
		case <-ctx.Done():
			return
		}
	}
}

func (wc *DealsWorker) startFetching(ctx context.Context) {
	results, err := wc.bitrixRepo.BitrixRepo.GettingBitrixUsersTrueResults(ctx)
	if err != nil {
		wc.logger.Error("failed to fetch bitrix users true-results", zap.Error(err))
		return
	}

	for _, result := range results {
		userPrivileges, err := wc.bitrixRepo.AccountRepo.GetPrivilegesByAccountID(ctx, result.QuizAccountID)
		if err != nil {
			wc.logger.Error("error getting user privileges", zap.Error(err))
			return
		}

		if !utils.VerifyUserPrivileges(userPrivileges) {
			wc.logger.Info("User don't have active quizCnt or quizUnlim privileges, aborting")
			continue
		}

		allAnswers, err := wc.bitrixRepo.AnswerRepo.GetAllAnswersByQuizID(ctx, result.Session)
		if err != nil {
			wc.logger.Error("error getting all user answers by result session", zap.Error(err))
			return
		}

		// todo
		//userTags, err := wc.amoRepo.AmoRepo.GetUserTagsByID(ctx, result.AmoAccountID)
		//if err != nil {
		//	wc.logger.Error("error getting user tags by ano account id", zap.Error(err))
		//	return
		//}'

		performerIDInt, err := strconv.Atoi(result.PerformerID)
		if err != nil {
			wc.logger.Error("error parsing performerID", zap.Error(err))
			return
		}

		var request interface{}

		if result.LeadFlag {
			request = models.CreatingLeadReq{
				Fields: models.CreateLeadFields{
					Title:        fmt.Sprintf("lead quiz number %d", result.QuizID),
					Opened:       "Y",
					AssignedByID: performerIDInt,
					SourceID:     result.SourceID,
					StatusID:     result.StatusID,
					UtmSource:    result.UTMs["utm_source"],
					UtmCampaign:  result.UTMs["utm_campaign"],
					UtmContent:   result.UTMs["utm_content"],
					UtmMedium:    result.UTMs["utm_medium"],
					UtmTerm:      result.UTMs["utm_term"],
				},
			}
		} else {
			request = models.CreatingDealReq{
				Fields: models.CreateDealFields{
					Title:        fmt.Sprintf("deal quiz number %d", result.QuizID),
					TypeID:       result.TypeID,
					StageID:      result.StageID,
					Opened:       "Y",
					AssignedByID: performerIDInt,
					CategoryID:   result.PipelineID,
					SourceID:     result.SourceID,
					UtmSource:    result.UTMs["utm_source"],
					UtmCampaign:  result.UTMs["utm_campaign"],
					UtmContent:   result.UTMs["utm_content"],
					UtmMedium:    result.UTMs["utm_medium"],
					UtmTerm:      result.UTMs["utm_term"],
				},
			}
		}

		reqMap, err := wc.constructField(ctx, request, allAnswers, result, performerIDInt)
		if err != nil {
			wc.logger.Error("error construct fields", zap.Error(err))
			return
		}

		wc.logger.Info("NOW DEAL/LEAD CONSTRUCTED IS:", zap.Any("DEAL", reqMap))

		var errCreating error
		var id int32

		if result.LeadFlag {
			id, errCreating = wc.bitrixClient.CreatingLead(reqMap, result.AccessToken, result.SubDomain)
			if errCreating != nil {
				wc.logger.Error("error creating lead", zap.Error(errCreating))
			}

		} else {
			id, errCreating = wc.bitrixClient.CreatingDeal(reqMap, result.AccessToken, result.SubDomain)
			if errCreating != nil {
				wc.logger.Error("error creating deal", zap.Error(err))
			}
		}

		if errCreating != nil {
			err = wc.bitrixRepo.BitrixRepo.SaveDealBitrixStatus(ctx, bitrix.SaveDealBitrixDeps{
				DealID:      0,
				AnswerID:    result.AnswerID,
				AccessToken: result.AccessToken,
				Status:      err.Error(),
			})
			if err != nil {
				wc.logger.Error("error saving deal status", zap.Error(err))
				return
			}
			return
		}

		fmt.Println("DEALID", id)

		err = wc.bitrixRepo.BitrixRepo.SaveDealBitrixStatus(ctx, bitrix.SaveDealBitrixDeps{
			DealID:      id,
			AnswerID:    result.AnswerID,
			AccessToken: result.AccessToken,
			Status:      "OK",
		})
		if err != nil {
			wc.logger.Error("error saving deal status", zap.Error(err))
			return
		}
	}
}

// todo ОБДУМАТЬ ЛИДЫ ПОЧЕМУ ЛИД ПРЕВРАЩАЕТСЯ В СДЕЛКУ
func (wc *DealsWorker) constructField(ctx context.Context, request interface{}, allAnswers []model.ResultAnswer, result model.BitrixUsersTrueResults, performerIDInt int) (map[string]map[string]interface{}, error) {
	// id поля - ответ по типу поля
	entityFieldsMap := make(map[model.FieldsType]map[string]string)
	entityFieldsMap[model.FieldTypeDeal] = make(map[string]string)
	entityFieldsMap[model.FieldTypeLead] = make(map[string]string)
	entityFieldsMap[model.FieldTypeCompany] = make(map[string]string)
	entityFieldsMap[model.FieldTypeContact] = make(map[string]string)

	// тип поля = правила на этот тип
	entityRules := make(map[model.FieldsType]map[int]string)
	entityRules[model.FieldTypeDeal] = result.FieldsRule.Deal.QuestionID
	entityRules[model.FieldTypeLead] = result.FieldsRule.Lead.QuestionID
	entityRules[model.FieldTypeCompany] = result.FieldsRule.Company.QuestionID
	entityRules[model.FieldTypeContact] = result.FieldsRule.Contact.QuestionID

	fmt.Println("result.FieldsRule.Deal.QuestionID", result.FieldsRule.Deal.QuestionID)
	fmt.Println("result.FieldsRule.Lead.QuestionID", result.FieldsRule.Lead.QuestionID)

	for entityType, rule := range entityRules {
		for _, data := range allAnswers {
			if fieldID, ok := rule[int(data.QuestionID)]; ok {
				fieldData, err := wc.bitrixRepo.BitrixRepo.GetFieldByID(ctx, fieldID)
				if err != nil {
					if err == sql.ErrNoRows {
						wc.logger.Info("This field id does not exist in db", zap.Any("fieldID", fieldID))
						continue
					}
					return nil, err
				}

				if fieldData.FieldType == model.FileCustomFieldsType {
					continue
				}

				//if entityType != model.FieldTypeDeal {
				if fieldData.FieldType == model.StringCustomFieldsType {
					content := strings.ReplaceAll(data.Content, " ", "")
					if content == "" {
						data.Content = "Пустая строка"
					}
					entityFieldsMap[entityType][fieldData.FieldName] = tools.EmojiUnicode(data.Content)
					continue
				}
				//}
				// todo need test // "https://storage.yandexcloud.net/squizanswer/5873c188-9ece-4540-96d8-4c3cb451a13f/109965/coq1ei7ot84c73fnk5cg.pdf"
				//fieldData.FieldType = model.FileCustomFieldsType
				if fieldData.FieldType == model.FileCustomFieldsType && data.Content != "" {
					fileName, base64Data, err := wc.bitrixClient.DownLoadFile(data.Content)
					if err != nil {
						return nil, err
					}

					fileEntry := models.FileField{
						FileData: []string{fileName, base64Data},
					}

					fileEntryJson, err := json.Marshal(fileEntry)
					if err != nil {
						return nil, err
					}

					//entityFieldsMap[entityType]["UF_CRM_1729686490"] = string(fileEntryJson)
					entityFieldsMap[entityType][fieldData.FieldName] = string(fileEntryJson)

					continue
				}
			}
		}
	}

	contactFields := entityFieldsMap[model.FieldTypeContact]
	companyFields := entityFieldsMap[model.FieldTypeCompany]

	dealFields := make(map[string]string)
	var dealReq models.CreatingDealReq
	leadFields := make(map[string]string)
	var leadReq models.CreatingLeadReq

	if result.LeadFlag {
		leadReq = request.(models.CreatingLeadReq)
		leadFields = entityFieldsMap[model.FieldTypeLead]
	} else {
		dealFields = entityFieldsMap[model.FieldTypeDeal]
		dealReq = request.(models.CreatingDealReq)
	}

	var resultInfo model.ResultContent
	err := json.Unmarshal([]byte(result.Content), &resultInfo)
	if err != nil {
		return nil, err
	}
	contactRuleMap := result.FieldsRule.Contact.ContactRuleMap

	contactFields, err = wc.addContactFields(ctx, contactFields, resultInfo.Name, model.TypeContactName, contactRuleMap)
	if err != nil {
		wc.logger.Error("error getting field data in addContactFields --- Name", zap.Error(err))
		return nil, err
	}
	if len(resultInfo.Phone) > 4 || resultInfo.Phone != "" {
		contactFields, err = wc.addContactFields(ctx, contactFields, resultInfo.Phone, model.TypeContactPhone, contactRuleMap)
		if err != nil {
			wc.logger.Error("error getting field data in addContactFields --- Phone", zap.Error(err))
			return nil, err
		}
	}
	contactFields, err = wc.addContactFields(ctx, contactFields, resultInfo.Text, model.TypeContactText, contactRuleMap)
	if err != nil {
		wc.logger.Error("error getting field data in addContactFields --- Text", zap.Error(err))
		return nil, err
	}
	contactFields, err = wc.addContactFields(ctx, contactFields, resultInfo.Email, model.TypeContactEmail, contactRuleMap)
	if err != nil {
		wc.logger.Error("error getting field data in addContactFields --- Email", zap.Error(err))
		return nil, err
	}
	contactFields, err = wc.addContactFields(ctx, contactFields, resultInfo.Address, model.TypeContactAddress, contactRuleMap)
	if err != nil {
		wc.logger.Error("error getting field data in addContactFields --- Address", zap.Error(err))
		return nil, err
	}

	//check duplicate contacts
	var fields []string
	var contactID int32

	name := resultInfo.Name
	if name == "" {
		name = fmt.Sprintf("empty name, quiz %d, triggered by answer - %d", result.QuizID, result.AnswerID)
	}

	if len(resultInfo.Phone) > 4 || resultInfo.Phone != "" {
		fields = append(fields, resultInfo.Phone)
	}

	if resultInfo.Email != "" {
		fields = append(fields, resultInfo.Email)
	}

	existContactData, err := wc.bitrixRepo.BitrixRepo.GetExistingContactBitrix(ctx, result.AmoAccountID, fields)
	if err != nil && !errors.Is(err, pj_errors.ErrNotFound) {
		return nil, err
	}
	if errors.Is(err, pj_errors.ErrNotFound) || len(existContactData) == 0 {
		fmt.Println("NO CONTACT", contactFields)
		contactReq := models.CreateContactReq{
			Fields: models.ContactFields{
				Name:        name,
				TypeID:      result.TypeID,
				SourceID:    result.SourceID,
				Opened:      "Y",
				UtmSource:   result.UTMs["utm_source"],
				UtmCampaign: result.UTMs["utm_campaign"],
				UtmContent:  result.UTMs["utm_content"],
				UtmMedium:   result.UTMs["utm_medium"],
				UtmTerm:     result.UTMs["utm_term"],
			},
		}

		contactReqMap := models.FormattingToMap(&contactReq, contactFields)
		contactID, err = wc.bitrixClient.CreateContact(contactReqMap, result.AccessToken, result.SubDomain)
		if err != nil {
			wc.logger.Error("error creating contact", zap.Error(err))
			return nil, err
		}

		if len(resultInfo.Phone) > 4 || resultInfo.Phone != "" {
			_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
				AccountID: result.AmoAccountID,
				BitrixID:  contactID,
				Field:     resultInfo.Phone,
			})
			if err != nil {
				return nil, err
			}
		}

		if resultInfo.Email != "" {
			_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
				AccountID: result.AmoAccountID,
				BitrixID:  contactID,
				Field:     resultInfo.Email,
			})
			if err != nil {
				return nil, err
			}
		}
	} else if existContactData != nil && len(existContactData) > 0 {
		contactID, err = wc.chooseAndCreateContact(ctx, result, resultInfo, existContactData, contactFields, contactRuleMap)
		if err != nil {
			return nil, err
		}
	}

	companyReq := models.CompanyReq{
		Fields: models.CompanyFields{
			Title:        fmt.Sprintf("Компания %d", result.AnswerID),
			AssignedByID: performerIDInt,
			Opened:       "Y",
			UtmSource:    result.UTMs["utm_source"],
			UtmCampaign:  result.UTMs["utm_campaign"],
			UtmContent:   result.UTMs["utm_content"],
			UtmMedium:    result.UTMs["utm_medium"],
			UtmTerm:      result.UTMs["utm_term"],
			ContactID:    contactID,
		},
	}

	companyReqMap := models.FormattingToMap(&companyReq, companyFields)
	companyID, err := wc.bitrixClient.CreateCompany(companyReqMap, result.AccessToken, result.SubDomain)
	if err != nil {
		wc.logger.Error("error creating company", zap.Error(err))
		return nil, err
	}

	var reqMap map[string]map[string]interface{}

	if result.LeadFlag {
		fmt.Println("leadFields", leadFields)
		leadReq.Fields.CompanyID = companyID
		leadReq.Fields.ContactIDs = append(leadReq.Fields.ContactIDs, contactID)
		reqMap = models.FormattingToMap(&leadReq, leadFields)
	} else {
		fmt.Println("dealFields", dealFields)
		dealReq.Fields.CompanyID = companyID
		dealReq.Fields.ContactIDs = append(dealReq.Fields.ContactIDs, contactID)
		//dealFields["UF_CRM_1729778229491"] = "UF_CRM_1729778229491"
		reqMap = models.FormattingToMap(&dealReq, dealFields)
	}

	return reqMap, nil
}

func (wc *DealsWorker) addContactFields(ctx context.Context, contactFields map[string]string, fieldValue string, fieldType model.ContactQuizConfig, fieldMap map[string]string) (map[string]string, error) {
	if fieldValue != "" && fieldMap[string(fieldType)] != "" {
		fieldData, err := wc.bitrixRepo.BitrixRepo.GetFieldByID(ctx, fieldMap[string(fieldType)])
		if err != nil {
			return nil, err
		}
		contactFields[fieldData.FieldName] = fieldValue
	}
	return contactFields, nil
}

func (wc *DealsWorker) chooseAndCreateContact(ctx context.Context, result model.BitrixUsersTrueResults, resultInfo model.ResultContent, existingContacts map[int32][]model.ContactBitrix, contactFields map[string]string, contactRuleMap map[string]string) (int32, error) {
	var err error
	// 1 ищем контакт в котором совпадает и телефон и емайл
	if (len(resultInfo.Phone) > 4 || resultInfo.Phone != "") && resultInfo.Email != "" {
		phoneMatchedContacts := make(map[int32]bool)
		for _, contactVariants := range existingContacts {
			for _, contact := range contactVariants {
				if contact.Field == resultInfo.Phone {
					phoneMatchedContacts[contact.BitrixID] = true
				}
			}
		}
		for _, contactVariants := range existingContacts {
			for _, contact := range contactVariants {
				if contact.Field == resultInfo.Email {
					if _, ok := phoneMatchedContacts[contact.BitrixID]; ok {
						wc.logger.Info("нашлось телефон и емайл в бд, с одинаковым битрикс ид", zap.Any("битрикс ид", contact.BitrixID), zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
						return contact.BitrixID, nil
					}
				}
			}
		}

		var phoneContactID, emailContactID int32
		var phoneID int64 /*emailID*/
		for _, contactVariants := range existingContacts {
			for _, contact := range contactVariants {
				if contact.Field == resultInfo.Phone {
					phoneContactID = contact.BitrixID
					phoneID = contact.ID
				}
				if contact.Field == resultInfo.Email {
					emailContactID = contact.BitrixID
					//emailID = contact.ID
				}
			}
		}

		if phoneContactID != 0 && emailContactID != 0 && phoneContactID != emailContactID {
			wc.logger.Info("нашлось телефон и емайл в бд, но это пока разные контакты", zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
			// делаем обновление телефона там где уже есть email
			valuePhone := make(map[string]string)
			valuePhone, err = wc.addContactFields(ctx, valuePhone, resultInfo.Phone, model.TypeContactPhone, contactRuleMap)
			if err != nil {
				wc.logger.Error("error adding contact fields", zap.Error(err))
				return 0, err
			}
			name := resultInfo.Name
			if name == "" {
				name = fmt.Sprintf("empty name, quiz %d, triggered by answer - %d", result.QuizID, result.AnswerID)
			}

			contactUpdateReq := models.CreateContactReq{
				Fields: models.ContactFields{
					Name: name,
				},
			}
			contactUpdateReqMap := models.FormattingToMap(&contactUpdateReq, valuePhone)

			err = wc.bitrixClient.UpdateContact(contactUpdateReqMap, result.AccessToken, result.SubDomain, emailContactID)
			if err != nil {
				wc.logger.Error("error updating contact", zap.Error(err))
				return 0, err
			}

			err = wc.bitrixRepo.BitrixRepo.UpdateBitrixContact(ctx, phoneID, resultInfo.Phone, emailContactID)
			if err != nil {
				return 0, err
			}

			// todo пока без линковки
			//_, err = wc.amoClient.LinkedContactToContact([]models.LinkedContactReq{
			//	{
			//		ToEntityID:   emailContactID,
			//		ToEntityType: "contacts",
			//		//Metadata: struct {
			//		//	//CatalogID int  `json:"catalog_id"`
			//		//	//Quantity  int  `json:"quantity"`
			//		//	IsMain bool `json:"is_main"`
			//		//	//UpdatedBy int  `json:"updated_by"`
			//		//	//PriceID   int  `json:"price_id"`
			//		//}(struct {
			//		//	//CatalogID int
			//		//	//Quantity  int
			//		//	IsMain bool
			//		//	//UpdatedBy int
			//		//	//PriceID   int
			//		//}{IsMain: true}),
			//	},
			//}, result.SubDomain, result.AccessToken, phoneContactID)
			//if err != nil {
			//	return 0, err
			//}

			return emailContactID, nil
		}
	}

	// 2 ищем контакт только по телефону
	if len(resultInfo.Phone) > 4 || resultInfo.Phone != "" {
		for _, contactVariants := range existingContacts {
			for _, contact := range contactVariants {
				if contact.Field == resultInfo.Phone {
					// нашли контакт по телефону
					emailExists := false
					for _, variant := range existingContacts[contact.BitrixID] {
						if variant.Field != contact.Field {
							if variant.Field != "" {
								emailExists = true
								break
							}
						}
					}
					if !emailExists && resultInfo.Email != "" {
						// email пустой обновляем контакт добавляя email, если не пустой
						wc.logger.Info("нашлось телефон, емайл не пустой, а в бд пустой. обновляем контакт", zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
						valueEmail := make(map[string]string)
						valueEmail, err = wc.addContactFields(ctx, valueEmail, resultInfo.Email, model.TypeContactEmail, contactRuleMap)
						if err != nil {
							wc.logger.Error("error adding contact fields", zap.Error(err))
							return 0, err
						}
						name := resultInfo.Name
						if name == "" {
							name = fmt.Sprintf("empty name, quiz %d, triggered by answer - %d", result.QuizID, result.AnswerID)
						}

						contactUpdateReq := models.CreateContactReq{
							Fields: models.ContactFields{
								Name: name,
							},
						}
						contactUpdateReqMap := models.FormattingToMap(&contactUpdateReq, valueEmail)

						err = wc.bitrixClient.UpdateContact(contactUpdateReqMap, result.AccessToken, result.SubDomain, contact.BitrixID)
						if err != nil {
							wc.logger.Error("error updating contact", zap.Error(err))
							return 0, err
						}
						_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
							AccountID: result.AmoAccountID,
							BitrixID:  contact.BitrixID,
							Field:     resultInfo.Email,
						})
						if err != nil {
							return 0, err
						}
						return contact.BitrixID, nil
					}
					if emailExists && resultInfo.Email != "" {
						// email не пустой значит это новый контакт создаем если наш email тоже не пустой
						wc.logger.Info("нашлось телефон, емайл не пустой и в бд не пустой. создаем новый контакт", zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
						name := resultInfo.Name
						if name == "" {
							name = fmt.Sprintf("empty name, quiz %d, triggered by answer - %d", result.QuizID, result.AnswerID)
						}

						contactCreateReq := models.CreateContactReq{
							Fields: models.ContactFields{
								Name:        name,
								TypeID:      result.TypeID,
								SourceID:    result.SourceID,
								Opened:      "Y",
								UtmSource:   result.UTMs["utm_source"],
								UtmCampaign: result.UTMs["utm_campaign"],
								UtmContent:  result.UTMs["utm_content"],
								UtmMedium:   result.UTMs["utm_medium"],
								UtmTerm:     result.UTMs["utm_term"],
							},
						}

						contactCreateReqMap := models.FormattingToMap(&contactCreateReq, contactFields)

						contactID, err := wc.bitrixClient.CreateContact(contactCreateReqMap, result.AccessToken, result.SubDomain)
						if err != nil {
							wc.logger.Error("error creating contact", zap.Error(err))
							return 0, err
						}

						_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
							AccountID: result.AmoAccountID,
							BitrixID:  contactID,
							Field:     resultInfo.Phone,
						})
						if err != nil {
							return 0, err
						}

						_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
							AccountID: result.AmoAccountID,
							BitrixID:  contactID,
							Field:     resultInfo.Email,
						})
						if err != nil {
							return 0, err
						}
						return contactID, nil
					}
					// если пустой то это нужный контакт возвращаем его id, так как если мейл пустой у нас но номер совпадает а в бд не пустой значит оно нам надо
					wc.logger.Info("нашлось телефон, емайл пустой возвращаем существующий контакт", zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
					return contact.BitrixID, nil
				}
			}
		}
	}

	// 3 ищем контакт только по email
	if resultInfo.Email != "" {
		for _, contactVariants := range existingContacts {
			for _, contact := range contactVariants {
				if contact.Field == resultInfo.Email {
					// нашли контакт по email
					phoneExists := false
					for _, variant := range existingContacts[contact.BitrixID] {
						if variant.Field != contact.Field {
							if variant.Field != "" {
								phoneExists = true
								break
							}
						}
					}
					if !phoneExists && (len(resultInfo.Phone) > 4 || resultInfo.Phone != "") {
						// телефон пустой обновляем контакт добавляя телефон, если не пустой
						wc.logger.Info("нашлось емайл, телефон не пустой, а в бд пустой. обновляем контакт", zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
						valuePhone := make(map[string]string)
						valuePhone, err = wc.addContactFields(ctx, valuePhone, resultInfo.Phone, model.TypeContactPhone, contactRuleMap)
						if err != nil {
							wc.logger.Error("error adding contact fields", zap.Error(err))
							return 0, err
						}
						name := resultInfo.Name
						if name == "" {
							name = fmt.Sprintf("empty name, quiz %d, triggered by answer - %d", result.QuizID, result.AnswerID)
						}

						contactUpdateReq := models.CreateContactReq{
							Fields: models.ContactFields{
								Name: name,
							},
						}
						contactUpdateReqMap := models.FormattingToMap(&contactUpdateReq, valuePhone)

						err := wc.bitrixClient.UpdateContact(contactUpdateReqMap, result.AccessToken, result.SubDomain, contact.BitrixID)
						if err != nil {
							wc.logger.Error("error updating contact", zap.Error(err))
							return 0, err
						}

						_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
							AccountID: result.AmoAccountID,
							BitrixID:  contact.BitrixID,
							Field:     resultInfo.Phone,
						})
						if err != nil {
							return 0, err
						}
						return contact.BitrixID, nil
					}
					if phoneExists && (len(resultInfo.Phone) > 4 || resultInfo.Phone != "") {
						// телефон не пустой значит это новый контакт создаем если наш телефон не пустой
						wc.logger.Info("нашлось емайл, телефон не пустой и в бд не пустой. создаем новый контакт", zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
						name := resultInfo.Name
						if name == "" {
							name = fmt.Sprintf("empty name, quiz %d, triggered by answer - %d", result.QuizID, result.AnswerID)
						}

						contactCreateReq := models.CreateContactReq{
							Fields: models.ContactFields{
								Name:        name,
								TypeID:      result.TypeID,
								SourceID:    result.SourceID,
								Opened:      "Y",
								UtmSource:   result.UTMs["utm_source"],
								UtmCampaign: result.UTMs["utm_campaign"],
								UtmContent:  result.UTMs["utm_content"],
								UtmMedium:   result.UTMs["utm_medium"],
								UtmTerm:     result.UTMs["utm_term"],
							},
						}

						contactCreateMapReq := models.FormattingToMap(&contactCreateReq, contactFields)

						contactID, err := wc.bitrixClient.CreateContact(contactCreateMapReq, result.AccessToken, result.SubDomain)
						if err != nil {
							wc.logger.Error("error creating contact", zap.Error(err))
							return 0, err
						}

						_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
							AccountID: result.AmoAccountID,
							BitrixID:  contactID,
							Field:     resultInfo.Phone,
						})
						if err != nil {
							return 0, err
						}

						_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
							AccountID: result.AmoAccountID,
							BitrixID:  contactID,
							Field:     resultInfo.Email,
						})
						if err != nil {
							return 0, err
						}
						return contactID, nil
					}

					// если пустой то это нужный контакт возвращаем его id, так как если телефон пустой у нас но мейл совпадает а в бд не пустой значит оно нам надо
					wc.logger.Info("нашлось емайл, телефон пустой возвращаем существующий контакт", zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
					return contact.BitrixID, nil
				}
			}
		}
	}

	wc.logger.Info("ничего не нашлось, создаем новый контакт", zap.String("Name", resultInfo.Name), zap.String("Phone", resultInfo.Phone), zap.String("Email", resultInfo.Email))
	// если дошлю до сюда то это новый контакт с новым email and phone
	name := resultInfo.Name
	if name == "" {
		name = fmt.Sprintf("empty name, quiz %d, triggered by answer - %d", result.QuizID, result.AnswerID)
	}

	contactCreateReq := models.CreateContactReq{
		Fields: models.ContactFields{
			Name:        name,
			TypeID:      result.TypeID,
			SourceID:    result.SourceID,
			Opened:      "Y",
			UtmSource:   result.UTMs["utm_source"],
			UtmCampaign: result.UTMs["utm_campaign"],
			UtmContent:  result.UTMs["utm_content"],
			UtmMedium:   result.UTMs["utm_medium"],
			UtmTerm:     result.UTMs["utm_term"],
		},
	}

	contactCreateMapReq := models.FormattingToMap(&contactCreateReq, contactFields)

	contactID, err := wc.bitrixClient.CreateContact(contactCreateMapReq, result.AccessToken, result.SubDomain)
	if err != nil {
		wc.logger.Error("error creating contact", zap.Error(err))
		return 0, err
	}

	if len(resultInfo.Phone) > 4 || resultInfo.Phone != "" {
		_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
			AccountID: result.AmoAccountID,
			BitrixID:  contactID,
			Field:     resultInfo.Phone,
		})
		if err != nil {
			return 0, err
		}
	}

	if resultInfo.Email != "" {
		_, err = wc.bitrixRepo.BitrixRepo.InsertContactBitrix(ctx, model.ContactBitrix{
			AccountID: result.AmoAccountID,
			BitrixID:  contactID,
			Field:     resultInfo.Email,
		})
		if err != nil {
			return 0, err
		}
	}
	return contactID, nil
}

func (wc *DealsWorker) Stop(_ context.Context) error {
	return nil
}
